// src/helptips/onboardingcompletetip.js
import React, { useEffect, useRef } from "react";
import { useDispatch, useSelector } from "react-redux";
import { CheckCircle as CheckCircleIcon } from "@mui/icons-material";
import { Button } from "@mui/material";
import { withStyles } from "tss-react/mui";
import { Loader, MenderTooltipClickable } from "@northern.tech/common-ui";
import { DEVICE_STATES, onboardingSteps } from "@northern.tech/store/constants";
import { getDemoDeviceAddress } from "@northern.tech/store/selectors";
import { getDeviceById, getDevicesByStatus, setOnboardingComplete } from "@northern.tech/store/thunks";
import { jsx, jsxs } from "react/jsx-runtime";
var CompletionButton = withStyles(Button, ({ palette }) => ({
  root: {
    backgroundColor: palette.background.default,
    "&:hover": {
      backgroundColor: palette.background.default
    }
  }
}));
var OnboardingCompleteTip = ({ anchor, targetUrl }) => {
  const timer = useRef();
  const dispatch = useDispatch();
  const url = useSelector(getDemoDeviceAddress) || targetUrl;
  useEffect(() => {
    dispatch(getDevicesByStatus({ status: DEVICE_STATES.accepted })).unwrap().then((tasks) => {
      return Promise.all(tasks[tasks.length - 1].deviceAccu.ids.map((id) => dispatch(getDeviceById(id))));
    }).finally(() => {
      timer.current = setTimeout(() => dispatch(setOnboardingComplete(true)), 12e4);
    });
    return () => {
      clearTimeout(timer.current);
    };
  }, [dispatch]);
  return /* @__PURE__ */ jsx(
    MenderTooltipClickable,
    {
      className: "tooltip onboard-icon onboard-tip",
      id: onboardingSteps.DEPLOYMENTS_PAST_COMPLETED,
      onboarding: true,
      startOpen: true,
      style: anchor,
      PopperProps: { style: { marginLeft: -30, marginTop: -20 } },
      title: /* @__PURE__ */ jsxs("div", { className: "content", children: [
        /* @__PURE__ */ jsxs("b", { children: [
          /* @__PURE__ */ jsx("p", { children: "Fantastic! You completed your first deployment!" }),
          /* @__PURE__ */ jsx("p", { children: "Your deployment is finished and your device is now running the updated software." })
        ] }),
        /* @__PURE__ */ jsxs("div", { className: "margin-bottom-small margin-top-small", children: [
          !url ? /* @__PURE__ */ jsx(Loader, { show: true }) : /* @__PURE__ */ jsx(CompletionButton, { className: "button", variant: "text", href: `${url}/index.html?source=${encodeURIComponent(window.location)}`, target: "_blank", children: `Go to ${url}` }),
          /* @__PURE__ */ jsx("br", {}),
          "and you should see the demo web application actually being run on the device."
        ] }),
        /* @__PURE__ */ jsx("p", { children: "NOTE: if you have local network restrictions, you may need to check them if you have difficulty loading the page." }),
        /* @__PURE__ */ jsxs("div", { className: "flexbox", children: [
          /* @__PURE__ */ jsx("div", { style: { flexGrow: 1 } }),
          /* @__PURE__ */ jsx(Button, { variant: "contained", color: "secondary", onClick: () => dispatch(setOnboardingComplete(true)), children: "Close" })
        ] })
      ] }),
      children: /* @__PURE__ */ jsx(CheckCircleIcon, {})
    }
  );
};
var onboardingcompletetip_default = OnboardingCompleteTip;

export {
  CompletionButton,
  OnboardingCompleteTip,
  onboardingcompletetip_default
};
//# sourceMappingURL=chunk-4FAJJ4WG.js.map